@@ -278,13 +278,13 @@ async def test_put_image(
     await registry_v2_image_source.put_image(
         image_config=response.image_config,
         image_name=image_name,
-        layer_files=response.compressed_layer_files,
+        layer_files=response.layer_files,
         manifest=response.manifest,
         manifest_list=response.manifest_list,
     )
 
-    for file in response.compressed_layer_files + response.uncompressed_layer_files:
-        file.close()
+    for layer_file in response.layer_files:
+        layer_file.close()
 
 
 @pytest.mark.online_modification
@@ -392,11 +392,8 @@ async def test_sign_image_same_image_source(
     )
     assertions(response)
 
-    for file in (
-        response.verify_image_data.compressed_layer_files
-        + response.verify_image_data.uncompressed_layer_files
-    ):
-        file.close()
+    for layer_file in response.verify_image_data.layer_files:
+        layer_file.close()
 
     # TODO: Test signing image twice (with same key, with different keys ...)
     #       Can we do this here (using dockerhub), or do we need to do this in test_imageconfig.py???
@@ -420,9 +417,7 @@ async def test_verify_image_integrity(
         manifest = result.manifest
         assert manifest
 
-        assert len(result.compressed_layer_files) == len(
-            result.uncompressed_layer_files
-        )
+        assert result.layer_files
 
     # 1. Unsigned
     response = await registry_v2_image_source.verify_image_integrity(
@@ -430,13 +425,14 @@ async def test_verify_image_integrity(
     )
     assertions(response)
 
-    for file in response.compressed_layer_files + response.uncompressed_layer_files:
-        file.close()
+    for layer_file in response.layer_files:
+        layer_file.close()
 
     # TODO: Test integrity on a signed image ...
     #       Can we do this here (using dockerhub), or do we need to do this in test_imageconfig.py???
 
 
+@pytest.mark.online
 async def test_verify_image_signatures(
     registry_v2_image_source: RegistryV2, known_good_image: TypingKnownGoodImage
 ):
@@ -487,3 +483,15 @@ async def test_verify_image_signatures(
 
     # Restore the original class method
     Signer.for_signature = original_method
+
+
+@pytest.mark.online
+async def test_issue_30():
+    """Test issue #30."""
+    image_name = ImageName.parse(
+        "tsaarni/uncompressed-layer-demo:latest@sha256:cb7e1611d9ff0e9090a13c5bdf343bb0cde9a8e30e60c9892e47c83a4ac97634"
+    )
+    # Note: Using default credentials store from the test environment
+    async with RegistryV2() as registry_v2_image_source:
+        response = await registry_v2_image_source.verify_image_integrity(image_name=image_name)
+        assert response
